package com.m20.absolute.schematron.ast;

import static org.junit.Assert.assertEquals;

import java.util.Arrays;

import org.junit.Test;

import com.m20.absolute.schematron.ast.xpath.BinaryPredicate;
import com.m20.absolute.schematron.ast.xpath.Literal;
import com.m20.absolute.schematron.ast.xpath.Path;
import com.m20.absolute.schematron.ast.xpath.Step;

/**
 * The class implements a JUnit test class. It was used for testing whether the
 * serialiser for Schematron outputs the rule in correct Schematron format.
 * 
 * @author Adriana Alexandru
 */

public class SchematronSerialiserTest {

	@Test
	public void testSerialise() throws Exception {
		Rule rule = new Rule();
		rule.setContext(Arrays.asList(new String[] { "//trade" }));
		rule.setId("r1");

		Path path = new Path();
		path.addStep(new Step("id"));
		BinaryPredicate test = new BinaryPredicate();
		test.setLeft(path);
		test.setOp(BinaryPredicate.Operator.EQUAL);
		test.setRight(new Literal("test"));

		Assertion assertion = new Assertion(test);
		rule.setAssertion(assertion);

		String result = SchematronSerialiser.serialise(rule);

		System.out.println(result);

		assertEquals("Missing opening rule tag", true, result.contains("<rule"));
		assertEquals("Context is incorrect", true,
				result.contains("context=\"//trade\""));
		assertEquals("ID is incorrect", true, result.contains("id=\"r1\""));
		assertEquals("Assertion is incorrect", true,
				result.contains("<assert test=\"id = 'test'\"/>"));
		assertEquals("Missing closing rule tag", true,
				result.contains("</rule>"));
	}

}
