package com.m20.absolute.schematron.ast;

import java.io.StringWriter;
import java.util.List;

import org.w3c.dom.*;

import com.m20.absolute.schematron.ast.xpath.XPathSerialiser;

import javax.xml.parsers.*;

import javax.xml.transform.*;
import javax.xml.transform.dom.*;
import javax.xml.transform.stream.*;

/**
 * The class implements the serialiser methods used to format the way Schematron
 * rules are displayed.The serialise method takes an instance of a rule as
 * argument and returns the rule as a string in the Schematron format.
 * 
 * @author Adriana Alexandru
 */

public class SchematronSerialiser {
	public static String serialise(Rule rule) throws Exception {
		// Create the Context string
		List<String> context = rule.getContext();
		String contextStr = "";
		if (context.isEmpty())
			return "";
		else {
			for (int i = 0; i < context.size(); i++) {
				contextStr = contextStr + context.get(i);
				if (i < context.size() - 1)
					contextStr = contextStr + " | ";
			}
		}
		// Get rule's id
		String id = rule.getId();
		// Create the Assertion's Test string
		String test = XPathSerialiser.serialise(rule.getAssertion().getTest());
		// Create XML
		DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
		DocumentBuilder docBuilder;
		docBuilder = dbfac.newDocumentBuilder();

		Document doc = docBuilder.newDocument();

		Element ruleElement = doc.createElement("rule");
		doc.appendChild(ruleElement);
		ruleElement.setAttribute("context", contextStr);
		ruleElement.setAttribute("id", id);

		Element assertionElement = doc.createElement("assert");
		assertionElement.setAttribute("test", test);
		ruleElement.appendChild(assertionElement);

		TransformerFactory transfac = TransformerFactory.newInstance();
		Transformer trans = transfac.newTransformer();
		trans.setOutputProperty(OutputKeys.OMIT_XML_DECLARATION, "yes");
		trans.setOutputProperty(OutputKeys.INDENT, "yes");
		trans.setOutputProperty(OutputKeys.ENCODING, "utf-8");
		trans.setOutputProperty("{http://xml.apache.org/xslt}indent-amount",
				"2");

		StringWriter sw = new StringWriter();
		StreamResult result = new StreamResult(sw);
		DOMSource source = new DOMSource(doc);
		trans.transform(source, result);
		String xmlString = sw.toString();

		xmlString = xmlString.replaceAll("&gt;", ">");
		xmlString = xmlString.replaceAll("&lt;", ">");

		System.out.println(xmlString);
		return xmlString;
	}
}
