package com.m20.absolute.schematron.ast.xpath;

import java.util.List;

/**
 * The class implements the serialiser methods used to format the way expressions are
 * displayed. Each serialise method takes an expression as argument and returns a string.
 * 
 * @author Adriana Alexandru
 */

public class XPathSerialiser {

	public static String serialise(PathExpr expression) {
		if (expression instanceof Literal) {
			return ((Literal) expression).getValue();
		} else if (expression instanceof Number) {
			return ((Number) expression).getValue().toString();
		} else if (expression instanceof FunctionCall) {
			return ((FunctionCall) expression).getName()
					+ serialise(((FunctionCall) expression).getParameters());
		} else if (expression instanceof ArithExpr) {
			return serialise(((ArithExpr) expression));
		} else if (expression instanceof IfStatem) {
			return serialise((IfStatem) expression);
		} else if (expression instanceof Path) {
			return serialise((Path) expression);
		} else if (expression instanceof BinaryOperator) {
			return serialise((BinaryOperator) expression);
		}

		else if (expression instanceof BinaryPredicate) {
			return serialise(((BinaryPredicate) expression));
		} else
			throw new RuntimeException("Take a look at the Serialiser class.");
	}

	public static String serialise(IfStatem expression) {
		return "not(" + serialise(expression.getIfPart()) + ") or "
				+ serialise(expression.getThenPart());
	}

	public static String serialise(ArithExpr expression) {
		String op = "";

		switch (expression.getOp()) {
		case PLUS:
			op = " + ";
			break;
		case MINUS:
			op = " - ";
			break;
		case MOD:
			op = " mod ";
			break;
		case DIV:
			op = " div ";
			break;
		case TIMES:
			op = " * ";
			break;
		}
		String output = XPathSerialiser.serialise(expression.getLeft()) + op
				+ XPathSerialiser.serialise(expression.getRight());

		return output;
	}

	public static String serialise(BinaryPredicate expression) {
		String op = "";

		switch (expression.getOp()) {
		case EQUAL:
			op = " = ";
			break;
		case GREATER:
			op = " > ";
			break;
		case GREATER_OR_EQUAL:
			op = " >= ";
			break;
		case LESS:
			op = " < ";
			break;
		case LESS_OR_EQUAL:
			op = " <= ";
			break;
		case NOT_EQUAL:
			op = " != ";
			break;
		}
		String output = XPathSerialiser.serialise(expression.getLeft()) + op
				+ XPathSerialiser.serialise(expression.getRight());

		return output;
	}

	public static String serialise(BinaryOperator expression) {
		String op = "";

		switch (expression.getOp()) {
		case AND:
			op = " and ";
			break;
		case OR:
			op = " or ";
			break;
		case IFF:
			op = " = ";
			break;
		}
		String output = XPathSerialiser.serialise(expression.getLeft()) + op
				+ XPathSerialiser.serialise(expression.getRight());

		return output;
	}

	public static String serialise(List<PathExpr> expressions) {
		String temp = "";
		String output = "(";
		if (expressions.isEmpty())
			return "";
		else {
			for (int i = 0; i < expressions.size(); i++) {
				temp = serialise(expressions.get(i));
				output = output + temp;
				if (i < expressions.size() - 1)
					output = output + ",";
				else
					output = output + ")";
			}
		}
		return output;
	}

	public static String serialise(Path path) {
		String output = "";
		if (path.getSteps().isEmpty())
			return "";
		else {
			for (int i = 0; i < path.getSteps().size(); i++) {
				output = output + path.getSteps().get(i).getStepName();
				if (i < path.getSteps().size() - 1)
					output = output + "/";
			}
		}
		return output;

	}

}
